%FITNOISE - fit an autoregressive model to the model outcome
%  this function requires ARfit to be in the search path of MATLAB
%  ARfit is a freeware MATLAB package by Tapio Schneider (tapio@gps.caltech.edu)
%  which can be downloaded at http://www.gps.caltech.edu/~tapio/arfit/
%  The procedure fits the parameters of a 1st order autoregressive model (AR1).
%
%  Usage:
%  FITNOISE - fits the noise in the last run (if parameters are changed the
%  model is rerun)
%  FITNOISE -paranal - Analyses the last paranal run and makes a figure
%  TAUS=FITNOISE returns the characteristic periods to TAUS (no output on screen);
%  FITNOISE(VAR) - fits noise to the columns of matrix VAR
%
%  See also: REDNOISE
function Taus = fitnoise(A)
global g_Y g_grind t;
if ~exist('arfit.m','file')
   disp('FITNOISE needs ARfit to be on the search path of MATLAB');
   disp('ARfit is a freeware MATLAB package by Tapio Schneider (tapio@gps.caltech.edu)');
   disp('which can be downloaded at http://www.gps.caltech.edu/~tapio/arfit/');
   disp('Add also the path where you install ARfit to the MATLAB path');disp(' ');
   error('ARfit not found');
end;
if (nargin == 0)  | (ischar(A) & (A(1) == '-'))
   i_parcheck;
   if isnan(g_grind.tstep)
      error('FITNOISE needs a fixed time step, use SIMTIME to set the timestep');
   end;
   if (nargin == 1) & strcmpi(A, '-paranal')
      Taus = analparanal;
      return;
   end;
   N0 = i_initvar;
   if i_settingschanged(N0)
      i_ru(g_grind.odefile, t, g_grind.ndays, N0, 1);
   end;
   A = g_Y;
else
   g_grind.statevars = [];
end;
if nargout == 0
   disp('variable   autocorr   95%CL     period       95%CL');
end;
for i = 1:size(A, 2)
   [Aest, Aerr, Test, Terr] = ar1fit(A(:,i));
   if isempty(g_grind.statevars)
      nam = sprintf('Col. %0.2g', i);
   else
      nam = i_statevars_names(i);
   end;
   if nargout == 0
      fprintf('%6s %10.4f%10.4f  %10.2f%10.2f\n', nam, Aest, Aerr, Test, Terr);
   end;
   Ts(i) = Test;
end;
if nargout > 0
   Taus = Ts';
end;
function Taus = analparanal
%Analyse the last paranal session
global g_paranal;
i = 1;
k = 1;
while i < length(g_paranal.p);
   ps(k) = g_paranal.p(i);
   j = i;
   while (j < length(g_paranal.p)) & (ps(k) == g_paranal.p(j))
      j = j + 1;
   end;
   for m = 1:size(g_paranal.Y, 2)
      yy = g_paranal.Y(i:j - 1, m);
      [Aest, Aerr, Test, Terr] = ar1fit(yy);
      if nargout == 0
         fprintf('%g %10.4f%10.4f  %10.2f%10.2f\n', ps(k), Aest, Aerr, Test, Terr);
      end;
      Taus(k, m) = Test;
   end;
   k = k + 1;
   i = j;
end;
figure(999);
plot(ps, Taus);
function [Aest, Aerr, Test, Terr] = ar1fit(A)
try
   if max(isnan(A))
      Aest = NaN;
      Aerr = NaN;
      Test = NaN;
      Terr = NaN;
   else
      if (max(A) < 1E-20) & (min(A) > -0.1)
         Aest = 1;
         Aerr = 0;
         Test = Inf;
         Terr = 0;
      else
         [west, Aest, Cest, SBC, FPE, th] = arfit(A, 1, 1);
         [Aerr] = arconf(Aest, Cest, west, th);
         Test = 1 ./ (1 - Aest);
         Terr = 1 ./ (1 - Aerr);
      end;
   end;
catch
   err=lasterror;
   disp(['ARFIT could not fit noise: ' err.message]);
   Aest = NaN;
   Aerr = NaN;
   Test = NaN;
   Terr = NaN;
end;
