%MARBLEPLOT - calculate marble-in-a-cup potential plot (1D model)
%   In one dimensional models you can represent the stability of equilibria as
%   potential landscape with hills and valleys. The landscape can be generated by 
%   solving the potential function of the model:
%   dx/dt=f(x)
%   transform this model to the potential equation:
%   -dV/dx=f(x)
%   The simulation of this model (initial condition=0) gives the stability plot at several values of 
%   a parameter, to create a plot comparable as in Scheffer et al. 2001 
%   (Nature 413: 591-596). The current state is often represented by a marble, but one should imagine
%   that this marble is heavily damped and sliding through a greasy substance (Strogatz, 1994).
%
%   Usage:
%   MARBLEPLOT - the user is asked to enter the required information.
%   MARBLEPLOT NPOT PAR [MINPAR,MAXPAR] VAR [MINVAR, MAXPAR] - NPOT= the number of
%   stability lines, PAR = name of control parameter; [MINPAR, MAXVAR]= range of 
%   parameter; VAR = state variable; [MINVAR MAXVAR] range of state variable.
%
%   See also POTENTIAL, NULL, AX

%   Copyright 2011 WUR
%   Revision: 1.0.7 $ $Date: 27-Oct-2011 17:17:08 $
function marbleplot(npot, par, rangepar, statevar, rangestatevar,varzero)
global g_grind;
stemlength=3;
if nargin >= 5
   if nargin<6
      varzero='minpot';
   end;
   npot = i_checkstr(npot);
   rangepar = i_checkstr(rangepar);
   rangestatevar = i_checkstr(rangestatevar);
else
   if nargin < 2
      par = g_grind.xaxis.var;
      iX = i_getno(par);
      if (isempty(iX) | (~iX.ispar)) & (~isempty(g_grind.pars))
         par = g_grind.pars{1};
      end;
   end;
   if nargin < 4
      statevar = g_grind.xaxis.var;
      iX = i_getno(statevar);
      if isempty(iX) | iX.ispar
         statevar = i_statevars_names(1);
      end;
   end;
   if g_grind.statevars.dim>1
       error('This function can only work with one state variable models');
   end;
   answer={'5',par,sprintf('[%d %d]',g_grind.xaxis.lim),statevar,sprintf('[%d %d]',g_grind.yaxis.lim),'minpot'};
   prompt={'Number of potential lines','Parameter','Range of parameter','State variable',...
      'Range of state variable','Zero relative to (minpot,minvar,maxvar,state var value)'};
   if isfield(g_grind, 'marbleplot')
      answer = g_grind.marbleplot;
   end;
   answer = inputdlg(prompt, 'Marble-in-a-cup plot', 1, answer);
   g_grind.marbleplot = answer;
   npot = i_checkstr(answer{1});
   par = answer{2};
   rangepar = i_checkstr(answer{3});
   statevar = answer{4};
   rangestatevar = i_checkstr(answer{5});
   varzero=answer{6};
end;
oldxax = g_grind.xaxis;
oldyax = g_grind.yaxis;
oldzax = g_grind.zaxis;
oldpar = evalin('base', par);
try
   iX = i_getno(par);
   if isempty(iX) | ~iX.ispar
      error([par ' is no parameter']);
   end;
   iX = i_getno(statevar);
   if isempty(iX) | ~iX.isvar
      error([statevar ' is no state variable']);
   end;
   ax('x', par, rangepar);
   ax('y', statevar, rangestatevar);
   ax('z');
   null 100;
   legend off;
   set(get(gca,'children'),'linestyle','-')
   hold on;
   hfig = gcf;
   pots = {};
   xs = {};
   ps = {};
   maxpot = -9999;
   minpot = 9999;
   for i = 1:npot
      apar = rangepar(2) - (rangepar(2) - rangepar(1)) * (i - 1) / (npot - 1);
      assignin('base', par, apar);
      [x, pot] = potential;
      switch varzero
      case 'minpot'
         pot=pot-min(pot)+1;
      case 'minvar'
         pot=pot-pot(1)+1;
      case 'maxvar'
         pot=pot-pot(end)+1;
      otherwise
         vardif=abs(x-str2double(varzero));
         ii=find(vardif==min(vardif));
         pot=pot-pot(ii)+i;
      end;
      pots{i}=pot;
      xs{i} = x;
      ps{i} = ones(size(x)) * apar;
      m = max(pot);
      if m > maxpot
         maxpot = m;
      end;
      m = min(pot);
      if m < minpot
         minpot = m;
      end;
   end;
   offset = minpot * stemlength;
   minpatch = minpot - offset;
   hold on;
   plot3([rangepar(1), rangepar(1), rangepar(2), rangepar(2)], ...
      [rangestatevar(1),rangestatevar(2),rangestatevar(2),rangestatevar(1)],[0, 0, 0, 0],'k-');
   for i = 1:npot;
      figure(hfig);     
      apar = ps{i}(1);
      xx = [apar; ps{i}; apar];
      yy = [rangestatevar(1); xs{i}; rangestatevar(2)];
      zz = [minpatch; pots{i} - offset; minpatch];
      gr = 0.5 + (npot - i + 1) / 2 / (npot + 1);
      patch(xx, yy, zz, [gr, gr, gr]);
      hold on;
      plot3([apar, apar, apar, apar], [rangestatevar(1), rangestatevar(1), rangestatevar(2), rangestatevar(2)], ...
         [minpatch, 0, 0, minpatch], 'k--');
   end;
   set(gca,'DrawMode', 'fast');
   set(gca, 'View', [ - 68 20]);
   set(gca, 'Zlim', [0, maxpot - offset]);
   set(gca, 'ZTick', []);
   set(gca, 'ZTickLabel', [])
   box off;
   g_grind.xaxis = oldxax;
   g_grind.yaxis = oldyax;
   g_grind.zaxis = oldzax;
   assignin('base', par, oldpar);
catch
   err=lasterror;
   g_grind.xaxis = oldxax;
   g_grind.yaxis = oldyax;
   g_grind.zaxis = oldzax;
   assignin('base', par, oldpar);
   rethrow(lasterror);
end;
