%NULL - Nullclines 2D
%   Creates nullclines in the 2D phase plane (it is also allowed to have a
%   parameter on the x or y-axes).
%   If there is no variable on the y-axis, a one-dimensional variant is used
%   (ITERMAP or PLOTDIFF)
%    
%   Usage: 
%   NULL - creates nullclines with default accuracy
%   NULL N - creates nullclines using a grid of N x N points (default=50)
%
%   Remark:
%   NULL is also a MATLAB function to determine the null space of a matrix, for help on this function see:
%   help(fullfile(matlabroot,'toolbox','matlab','matfun','null'))
%   you can still use this function.
%   
%   
%   See also ITERMAP, PLOTDIFF, NULL3, AX, GROWTHS

%   Copyright 2011 WUR
%   Revision: 1.0.7 $ $Date: 27-Oct-2011 17:17:08 $
function Z=null(npoints, opt)
if (nargout>0)|((nargin>0)&(~ischar(npoints)) & (length(npoints)>1))
    %there is a problem that NULL was already defined in MATLAB,
    %if the first argument is a matrix or if there is output
    %it is assumed that the matlab NULL is meant.
    cur=pwd;
    cd(fullfile(matlabroot,'toolbox','matlab','matfun'));
    if nargin==2
        Z=null(npoints,opt);
    elseif nargin==1
        Z=null(npoints);
    else
        Z=null;
    end;
    cd(cur);
    return;
end;
global g_grind ;
i_parcheck;
findnull = 0;
if nargin == 0
   npoints = 50;
else
   if ischar(npoints) & strncmpi(npoints, '-f', 2)
      npoints = 50;
      findnull = 1;
   end;
   npoints = i_checkstr(npoints);
   if (nargin == 2) & ischar(opt) & strncmpi(opt, '-f', 2)
      findnull = 1;
   end;
end;
if findnull
   g_grind.findnull.ndx = []; %option to find the nullclines of other variables
   for i = 1:g_grind.statevars.dim
      nam = i_statevars_names(i);
      if ~strcmp(g_grind.xaxis.var, nam) & ~strcmp(g_grind.yaxis.var, nam)
         g_grind.findnull.ndx = [g_grind.findnull.ndx(:); i];
      end;
   end;
elseif isfield(g_grind, 'findnull')
   g_grind = rmfield(g_grind, 'findnull');
end;
iX = i_getno(g_grind.xaxis.var);
iY = i_getno(g_grind.yaxis.var);
if ~(iX.isvar|iX.ispar|iX.isext) | ~(iY.isvar|iY.ispar|iY.isext)
   if isempty(g_grind) | g_grind.solver.isdiffer
      itermap(1);
   else
      plotdiff;
   end;
   return;
end;
if (isempty(iX.no) | isempty(iY.no))
   ax('?');
   errordlg('Cannot create null-isoclines if there are no state variables on the axes, use "phas 2" instead.');
   error('null');
end
[X, Y, Vect] = i_vector(npoints, iX, g_grind.xaxis.lim, iY, g_grind.yaxis.lim, [],0);
[H, new] = i_makefig('phase2');
if new
   set(H, 'WindowButtonDown', 'i_callb(''mdown'')');
   set(H, 'WindowButtonMotionFcn', 'i_callb(''mmove'')');
end;
set(H, 'Name', 'Phase plane');
oldhold = ishold;
hold('on');
leg = cell(g_grind.statevars.dim, 1);
%Hs = zeros(g_grind.statevars.dim, 1);
colors = {'m', 'b', 'r', 'g','k'};
lines = {':',':.','--','-'};
maxi = size(colors, 2) * size(lines, 2);
nleg = 0;
for i = 1:g_grind.statevars.dim
   j = mod(i, maxi);
   pen = [char(lines{floor(j / size(colors, 2) + 1)}) char(colors{mod(j, size(colors, 2)) + 1})];
   if g_grind.version.isoctave %not compatible
       [c] = contour(X, Y, Vect(:, :, i), [0, 0]);
   %    set(h, 'linewidth', g_grind.pen.linewidth, 'linestyle',pen(1),'edgecolor',pen(2),'facecolor',pen(2)  );
   else
       [c, h] = contour(X, Y, Vect(:, :, i), [0, 0], pen);
       set(h, 'linewidth', g_grind.pen.linewidth);
   end;
   if ~isempty(c)
      nleg = nleg + 1;
      if g_grind.solver.isdiffer
         leg{nleg} = i_disptext([i_statevars_names(i) '_{t} = ' i_statevars_names(i) '_{t+1}']);
      else
         leg{nleg} = i_disptext([i_statevars_names(i) '''' '=0']);
      end;
      %Hs(nleg) = h(1);
   end;
end;
if new
   i_plotdefaults(gcf);
   set(gca, 'drawmode','fast');
   if nleg > 0
      leg = leg(1:nleg);
      legend(leg{:});
   end;
   h = xlabel(i_disptext(g_grind.xaxis.var));
   set(h, 'fontsize', g_grind.pen.fontsize);
   h = ylabel(i_disptext(g_grind.yaxis.var));
   set(h, 'fontsize', g_grind.pen.fontsize);
end;
if g_grind.statevars.dim > 2
   title(i_disptext(['Valid for ' i_othervars(i_initvar, iX.no, iY.no)]));
end;
set(gca, 'XLim', g_grind.xaxis.lim);
set(gca, 'YLim', g_grind.yaxis.lim);
if ~oldhold
   hold off;
end;
