%TIMESENS - Performs numerical sensitivity analysis of a parameter in time
%  This algorithm does two precise runs with a slightly different parameter
%  or initial condition. It computes the derivative during the run. The plots are
%  added to a time plot if possible.
%
%  Usage:
%  TIMESENS PAR - adds PAR in the time plots
%  TIMESENS PAR PAR2 PAR3 etc - analyses PAR1-3
%  TIMESENS -ALL - adds all parameters and state variables
%  TIMESENS -C - clear all data
%  TIMESENS -U - update all
%  TIMESENS -D 1E-10 sets disturbance to 1E-10 (default=1E-8)
%  TIMESENS -D PAR 1E-10 - sets only disturbance for PAR
%
%  See also TIME, SIMTIME, OUT

%   Copyright 2011 WUR
%   Revision: 1.0.7 $ $Date: 27-Oct-2011 17:17:09 $
function [result] = timesens(varargin)
global g_Y g_grind g_sens;
evalin('base','global g_sens');
defaultdisturb = 1E-8;
i_parcheck;
if nargin == 0
   answer=inputdlg({'Enter parameter:'},'timesens',1);
   if isempty(answer) & ~isempty(answer{1})
      return;
   end;
   varargin{1} = answer{1};
end;
par = varargin{1};
if ischar(par)
   if par(1) == '-'
      if strcmpi(par, '-u')
         doupdate(g_grind.ndays);
         return;
      elseif strcmpi(par, '-all')
         varargin=[g_grind.pars g_grind.statevars.names];
      elseif strcmpi(par, '-d')
         if length(varargin) == 2
            disturb = i_checkstr(varargin{2});
            for i = 1:length(g_sens.pars)
               g_sens.disturb(i) = disturb;
            end;
         else
            i = findadd(varargin{2});
            if i <= length(g_sens.pars)
               g_sens.disturb(i) = i_checkstr(varargin{3});
            end;
         end
         return;
      elseif strcmpi(par, '-c')
         if ~isempty(g_sens)
            for i = 1:length(g_grind.timevars)
               for k = 1:length(g_sens.pars)
                  for j = 1:length(g_grind.statevars.names)
                     out('-silent',sprintf('-%d',i),'-remove',sprintf('timesens(''d(%s)/d(%s)'')',...
                        g_grind.statevars.names{j}, g_sens.pars{k} ));
                  end;
               end;
            end;
         end;
         g_sens = [];
         out('-silent','-cleantimevars')
         disp('Cleared all');
         return;
      end;
   end;
   f1 = strfind(par, '(');
   if ~isempty(f1)
      f2 = strfind(par, ')');
      var = par(f1(1) + 1:f2(1) - 1);
      par1 = par(f1(2) + 1:f2(2) - 1);
      ivar = i_varno(var);
      if isempty(ivar)
         error('not a state variable');
      end;
      ipar = findadd(par1);
      if ~isfield(g_sens, 'Ys') | length(g_sens.Ys) < ipar
         addpar(par1, defaultdisturb);
         g_sens.lastsettings = [];
      end;
      if settingschanged
         doupdate(g_grind.ndays);
      end;
      if ~isfield(g_sens, 'disturb') | (length(g_sens.disturb) < ipar) | (g_sens.disturb(ipar)==0)
         g_sens.disturb(ipar) = defaultdisturb;
      end;
      yy = g_sens.Ys{ipar};
      result = (yy(:,ivar) - g_Y(:, ivar)) / g_sens.disturb(ipar);
      return;
   end;
end;
npar = 0; 
s='Added sensitivities of';
for j = 1:length(varargin)
   if addpar(varargin{j}, defaultdisturb)
      npar = npar + 1;
      s=sprintf('%s ',s,varargin{j});
   end;
end;
if npar > 0
   disp(s);
   disp('Run TIME to see the results');
   if strcmp(g_grind.solver.name, 'ode45')
      if g_grind.solver.opt.AbsTol > 1e-10 | g_grind.solver.opt.RelTol > 1e-10
         disp('timesens has set solver tolerances to values 1E-11');
         solver('ode45', 1E-11, 1E-11);
      end;
   end;
end;

function ok = addpar(par, disturb)
global g_grind g_sens;
no = i_getno(par);
ok = 1;
if isempty(no.no)
   fprintf('unknown parameter %s\n', par);
   ok = 0;
   return;
end;
i = findadd(par);
parexist = ~isempty(g_sens);
if parexist
   parexist=(i <= length(g_sens.pars));
end;
g_sens.pars{i} = par;
g_sens.disturb(i) = disturb;
if ~parexist
   g_sens.lastsettings = [];
   len = length(g_grind.timevars) + 1;
   for j = 1:length(g_grind.statevars.names)
      out('-silent',sprintf('-%d',len),'-add',sprintf('timesens(''d(%s)/d(%s)'')',...
         g_grind.statevars.names{j}, g_sens.pars{i}));
   end;
end;

function res = settingschanged
global g_sens g_t;
if isempty(g_sens) | ~isfield(g_sens, 'lastsettings') | isempty(g_sens.lastsettings)
   res = 1;
   return;
end;
settings = i_getsettings;
if ~isempty(g_t);
   settings(1) = g_t(end);
end;
res = isdifferent(settings, g_sens.lastsettings);

function res = isdifferent(A, B)
res=~min(size(A) == size(B));
if isempty(B) & isempty(A)
   res = 0;
elseif ~res
   compar = A == B;
   res = ~min(compar);
   if res
      res = ~min(compar + isnan(A) .* isnan(B));
   end;
end;

function doupdate(ndays)
global g_Y g_t t g_grind g_sens;
disp('updating timesens...');
g_sens.lastsettings = i_getsettings;
g_sens.lastsettings(1) = g_t(end);
oldY = g_Y;
oldt = g_t;
try
   for l_i = 1:length(g_sens.pars)
      oldpar = evalin('base', g_sens.pars{l_i});
      assignin('base',  g_sens.pars{l_i}, oldpar + g_sens.disturb(l_i));
      N0 = i_initvar;
      i_ru(g_grind.odefile, t, g_t(end), N0, 0);
      assignin('base', g_sens.pars{l_i}, oldpar);
      g_sens.Ys{l_i} = interp1(g_t, g_Y, oldt);
   end;
   g_Y = oldY;
   g_t = oldt;
catch
   err=lasterror;
   g_Y = oldY;
   g_t = oldt;
   rethrow(err);
end;

function i = findadd(par)
global g_sens;
i = 1;
if ~isempty(g_sens)
   while i <= length(g_sens.pars) & ~strcmp(g_sens.pars{i}, par)
      i = i + 1;
   end;
end;

